function [a, b,at,bt] = bloch(rf, g, r, gammabar,dt,B0)
% bloch: Bloch equation solver based on piece-wise constant solution of
% the Bloch equation
% rf: (Nt,1) vector, rf waveform
% g: (d,Nt) vector, gradient vector , d spatial dimension
% r: (Nr,d) space location vector
% gammabar: gyromagnetic ratio, Hz/T
% dt: time period
% B0: (Nr,1) vector, B0 inhomogeneity
% the convention of rotation here is right-hand convention

%% Initialization
L = size(rf,2);
Nt = size(rf,1);
Nr = size(r,1);
if size(rf,2) == 1
    rft = repmat(rf,1,Nr);
    b1x = real(rft);
    b1y = imag(rft);
else
    b1x = real(rf);
    b1y = imag(rf);
end


if size(g,2) == 1 % constant gradient
    g = repmat(g,1,Nt);
end

% default value of b0
if ~exist('B0','var') || isempty(B0)
    B0 = zeros(Nr,1);
end

% uniform dt by default
if length(dt) == 1
    dt = ones(Nt,1)*dt;
end

% calculate the final state only
a0 = ones(Nr,1);
b0 = zeros(Nr,1);
if nargout > 2
    at = zeros(Nr,Nt);
    bt = zeros(Nr,Nt);
end

% solve Bloch equation using piece-wise constant solution
% right-hand convention
n_dir = zeros(Nr,3);
if nargout < 4
    for indt = 1:Nt
        % calculate b1 field
        % rotation axis
        n_dir(:,1) = b1x(indt,:);
        n_dir(:,2) = b1y(indt,:);
        n_dir(:,3) = r*g(:,indt) + B0;
        norm_n_dir = sqrt(sum(n_dir.^2,2));
        % rotation angle
        theta = -2*pi*gammabar*dt(indt)*norm_n_dir;  % rotation angle (rad)
        % update ab
        n_dir = n_dir./(repmat(norm_n_dir,1,3)+eps);
        da = cos(theta/2)-1i*n_dir(:,3).*sin(theta/2);
        db = -1i*(n_dir(:,1)+1i*n_dir(:,2)).*sin(theta/2);
        a = da.*a0 - conj(db).*b0;
        b = db.*a0 + conj(da).*b0;
        a0 = a;
        b0 = b;
    end
else
    for indt = 1:Nt
        % calculate b1 field
        % rotation axis
        n_dir(:,1) = b1x(indt,:);
        n_dir(:,2) = b1y(indt,:);
        n_dir(:,3) = r*g(:,indt) + B0;
        norm_n_dir = sqrt(sum(n_dir.^2,2));
        % rotation angle
        theta = -2*pi*gammabar*dt(indt)*norm_n_dir;  % rotation angle (rad)
        % update ab
        n_dir = n_dir./(repmat(norm_n_dir,1,3)+eps);
        da = cos(theta/2)-1i*n_dir(:,3).*sin(theta/2);
        db = -1i*(n_dir(:,1)+1i*n_dir(:,2)).*sin(theta/2);
        a = da.*a0 - conj(db).*b0;
        b = db.*a0 + conj(da).*b0;
        a0 = a;
        b0 = b;
        % track the time varing of a and b
        at(:,indt) = a0;
        bt(:,indt) = b0;
    end
end
