%% MDSLR_90ex_circROI
% Design a 90 excitation pulse of a circular region of interest
% Created by Chao Ma
% Last modified: Dec-01-2013
%

clear all;
close all;
clc;

fignum = 300;               % figure number
gammabar = 4257.6;          % H/Gauss

% add path of CVX
if isunix
    addpath(genpath('./CVX_Unix'));
else
    addpath(genpath('.\CVX_Windows'));
end

%% Define the excitation pattern
% flip anlge and pulse type
fa = 90*pi/180;
pulseType = 'ex';
%Calculate the ripples with given effective ripple level
deltape = 0.05;
deltase = 0.05;
[deltap,deltas] = erp2rp(deltape,deltase,pulseType);
deltapx = deltap;
deltapz = deltap;
deltasx = deltas;
deltasz = deltas;

% Define the desired excitation pattern
Lx = 20;        % cm
Lz = 20;        % cm
ROIr = 3.5;       % radius of the region of interest
% Setup 
Nsx = 128;
x = (-Lx/2):(Lx/Nsx):(Lx/2-(Lx/Nsx));
Nsz = 128;
z = (-Lz/2):(Lz/Nsz):(Lz/2-(Lz/Nsz));
[X,Z] = meshgrid(x,z);
% Define the Desired excitation pattern
D = zeros(Nsz,Nsx);
if sin(fa/2)*(1+deltapx+deltapz) >1
    d = sin(fa/2)*(1-deltapx-deltapz);
else
    d = sin(fa/2);
end
D(X.^2+Z.^2<ROIr^2) = d;

% Determine the passband and stopband position along the z-axis
BWz = 2*ROIr/Lz;                               % normalized bandwidth
TBW = 5;                                       % time-bandwith product
Nz = round(TBW/BWz/2)*2 + 1;                  % number of sampling point
[Fpz,Fsz] = firfpfs(deltapz,deltasz,BWz,Nz);

% For each z, determine the passband and stopband position along the x-axis
indzLb = find(z>-ROIr,1,'first');
indzRb = find(z<ROIr,1,'last');
Nx  = 11;
Fpx = zeros(Nsz,1);
Fsx = zeros(Nsz,1);
dx = zeros(Nsz,1);
for indz = indzLb:indzRb
    dx(indz) = sqrt(ROIr^2 - z(indz)^2)*2;
    BWx = dx(indz)/Lx;
    [Fpx(indz),Fsx(indz)] = firfpfs(deltapx,deltasx,BWx,Nx);
end

%% Design gradient waveforms
gmax = 4;
smax = 13000;
dtG = 1e-5;

% Calculate z-gradient 
T = 0.6*1e-3;
temp = ceil(T/(Nz-1)/dtG);
T = (Nz-1)*temp*dtG;
dtRf = T/(Nz-1);
Nos = round(dtRf/dtG);
BW = TBW/T;
Gz = BW/(gammabar*2*ROIr);
% Build refocusing lobe
gzSub = Gz.*ones((Nz-1)*Nos/2,1);
gzSubRe= ramp_for_trap(gzSub,smax,dtG);
% positive slobe
gzSub = Gz.*ones(Nz*Nos,1);
[gzSub,gzSubPlateauInd] = ramp_for_trap(gzSub,smax,dtG);

% Calculate the gx blips
kstart = 0;
kend = 1/Lx*2*pi;
[~,gxSub] = excite_trapgrad(kstart,kend,gmax,smax,dtG);
[~,gxSubRe] = excite_trapgrad(kstart,-(Nx-1)/2*kend,gmax,smax,dtG);

%% Design the desired excitation profile for each subpulse
mdslrpara = [];
mdslrpara.D = D;
mdslrpara.Lx = Lx;
mdslrpara.Fpx = Fpx;
mdslrpara.Fsx = Fsx;
mdslrpara.Nx = Nx;
mdslrpara.Lz = Lz;
mdslrpara.dz = 2*ROIr;
mdslrpara.Fpz = Fpz;
mdslrpara.Fsz = Fsz;
mdslrpara.Nz = Nz;
mdslrpara.Gz = Gz;
mdslrpara.gammabar = gammabar;
mdslrpara.dt = dtRf;
mdslrpara.phmod = 1;
mdslrpara.delta = [deltapz,deltasz; deltapx,deltasx];
mdslrpara.display_on = 0;
B1Wf = mdslr( mdslrpara );

%% Concentrate gradient and RF pulse waveforms
% concentrate the gradient and rf pulse waveform
grfpara = [];
grfpara.gxSub = gxSub;
grfpara.gzSub = gzSub;
grfpara.gzSubPlateauInd = gzSubPlateauInd(:);
grfpara.gzSubRe = gzSubRe;
grfpara.B1Wf = B1Wf;
grfpara.dtG = dtG;
grfpara.dtRf = dtRf;
if strcmp(pulseType,'sta') || strcmp(pulseType,'ex')
    grfpara.gxSubRe = gxSubRe;
    grfpara.gzReOn = 2;
else
    grfpara.gzReOn = 0;
end
[ g,rf ] = sub_grf(grfpara);

%% Calculate the resulting excitation pattern using Bloch equation simulation
NsOs = 2;
Lx2 = 18;
Lz2 = 18;
Nsx2 = Nsx*NsOs;
x = (-Lx2/2):(Lx2/Nsx2):(Lx2/2-(Lx2/Nsx2));
Nsz2 = Nsz*NsOs;
z = (-Lz2/2):(Lz2/Nsz2):(Lz2/2-(Lz2/Nsz2));
[X,Z] = meshgrid(x(:),z(:));
XZ = [X(:),Z(:)];

% Bloch equation simulation
[aXZsim,bXZsim] = bloch(rf,g,XZ,gammabar,dtG);

%test the results
aXZsim = reshape(aXZsim,Nsx2,Nsz2);
bXZsim = reshape(bXZsim,Nsx2,Nsz2);
if strcmp(pulseType,'refocus')
    mxyXZsim = 1i*bXZsim.^2;
else
    mxyXZsim = 2.*conj(aXZsim).*bXZsim;
end
mzXZsim = 1 - 2.*abs(bXZsim).^2;

%% Display results
% Display waveform
t = (0:length(rf)-1)*dtG;
figure(fignum);
subplot(211);
plot(t*1000,g(2,:),'LineWidth',2); hold on;
plot(t*1000,g(1,:),'r','LineWidth',2);
xlabel('t (ms)','FontSize',20);
ylabel('g (G/cm)','FontSize',20);
axis([-0.05,10.0,-1.0,0.5]);
set(gca,'FontSize',20,'LineWidth',2);
subplot(212);
plot(t*1000,imag(rf),'LineWidth',2); hold on;
plot(t*1000,real(rf),'r','LineWidth',2);
xlabel('t (ms)','FontSize',20);
ylabel('rf (G)','FontSize',20);
axis([-0.05,10.0,-0.15,0.05]);
set(gca,'FontSize',20,'LineWidth',2);
fignum = fignum + 1;

% display simulated excitation pattern
tz = z(Nsz2/2+1)*ones(Nsz2,1);
tx = x(Nsz2/2+1)*ones(Nsz2,1);
figure(fignum);
imagesc(x,z,abs(mxyXZsim)); hold on;
plot(x,tz,'--r','LineWidth',2);
plot(tx,z,'--r','LineWidth',2);
colormap(gray(256));
axis image off;
fignum = fignum + 1;

% plot the profile
figure(fignum);
plot(x,abs(mxyXZsim(Nsx2/2+1,:)),'b','LineWidth',2); 
xlabel('x (cm)','FontSize',20);
ylabel('|m_{xy}|', 'FontSize',20);
axis([-10,10,0,1.05]);
set(gca,'FontSize',20,'LineWidth',2);
fignum = fignum + 1;

figure(fignum);
plot(z,abs(mxyXZsim(:,Nsz2/2+1)),'b','LineWidth',2); 
xlabel('z (cm)','FontSize',20);
ylabel('|m_{xy}|', 'FontSize',20);
axis([-10,10,0,1.05]);
set(gca,'FontSize',20,'LineWidth',2);
fignum = fignum + 1;
