function [k,g,s,t] = excite_trapgrad(kstart,kend,gmax,smax,dt)
% excite_trapgrad
% Trapezoidal (or triangular) gradient waveform for traversing from one
% point to another point with a straight line in the excitation k-space
% (can be 1D, 2D, or 3D), only work for inherently refocused trajecotry
% (start and end at origin)
% Input
%  kstart, kend: length 1, 2, or 3 arrays (for 1D, 2D, or 3D),
%  representing starting and ending k-space locations.
%  gmax: max gradient amplitude (gauss/cm)  [3.9]
%  smax: max slew rate (gauss/cm/sec)  [14500]
%  dt: time interval between two neighboring samples (sec)  [4e-6]
% Output
%  k: Nxnd complex matrix for k-space samples (rad/cm), where N is
%  the number of points, nd is number of dimensions
%  g: Nxnd complex matrix for gradient waveform (Gauss/cm)
%  s: Nxnd complex matrix for slew rate waveform (Gauss/cm/sec)
%  t: Nx1 real vector for time points

if nargin < 3
    gmax = 3.9;
end

if nargin < 4
    smax = 14500;
end

if nargin < 5
    dt = 4e-6;
end

gamma = 4257.6;  % Hz/Gauss

[kjumpmax,indmax] = max(abs(kend-kstart));  % the "base" gradient waveform, other waveforms are designed by scaling
[kbase,gbase] = excite_trapgrad1d(kstart(indmax),kend(indmax),gmax,smax,dt);
N = length(gbase);

nd = length(kstart);  % dimension
k = zeros(N,nd);
g = zeros(N,nd);
s = zeros(N,nd);
for ind = 1:nd
    g(:,ind) = gbase*(kend(ind)-kstart(ind))/(kend(indmax)-kstart(indmax));
    k(:,ind) = 2*pi*gamma*cumsum(g(:,ind))*dt;  % (rad/cm)  % cannot use g2ks here because that will calculate cumsum(g), here we assume inherently refocus
    s(:,ind) = [0; diff(g(:,ind))/dt];  % Gauss/cm/sec
end
t = (0:dt:(N-1)*dt)';  % calculate time points
