function [k,g,s,t] = excite_trapgrad1d(kstart,kend,gmax,smax,dt)
% excite_trapgrad1d, primarily used for sub-function of excite_trapgrad,
% use excite_trapgrad for creating gradient waveforms
% Trapezoidal (or triangular) gradient waveform for traversing from one
% point to another point with a straight line in the excitation k-space
% (kx, ky, or kz, but only 1D), only work for inherently refocused
% trajecotry (start and end at origin)
% Input
%  kstart, kend: scalars representing starting and ending k-space locations,
%  along kx, ky, or kz, but only 1D.
%  gmax: max gradient amplitude (gauss/cm) [3.9]
%  smax: max slew rate (gauss/cm/sec) [14500]
%  dt: time interval between two neighboring samples (sec) [4e-6]
% Output
%  k: Nx1 complex vector for k-space samples (rad/cm), where N is
%  the number of points
%  g: Nx1 complex vector for gradient waveform (Gauss/cm)
%  s: Nx1 complex vector for slew rate waveform (Gauss/cm/sec)
%  t: Nx1 real vector for time points

if nargin < 3
    gmax = 3.9;
end

if nargin < 4
    smax = 14500;
end

if nargin < 5
    dt = 4e-6;
end

gamma = 4257.6;  % Hz/Gauss

area = (kend-kstart)/(2*pi*gamma);
if area == 0  % same point
    g = [];
    k = [];
    s = [];
    t = [];
elseif abs(area)<gmax^2/smax  % area is small so triangular shape is used
    T_ramp = sqrt(abs(area)/smax);  % ramp-up time
    N_ramp = ceil(T_ramp/dt);  % integer number of minimum time interval
    p = area/(N_ramp^2*dt);
    g = [0:p:(N_ramp-1)*p,N_ramp*p,(N_ramp-1)*p:-p:0]';
    N = 2*N_ramp+1;  % total number of points
else                          % area is large so trapezoidal shape is used
    T_ramp = gmax/smax;  % ramp-up time
    N_ramp = ceil(T_ramp/dt);
    N_plt = ceil((abs(area)-N_ramp*(N_ramp-1)*smax*dt^2)/(gmax*dt));  % number of points for plateau
    p = sign(area)*(abs(area)-N_plt*dt*gmax+2*dt*gmax)/(N_ramp*(N_ramp+1)*dt);  % actual increment for ramps, the 2 here comes from the first and last points on plateau to make sure they don't violate smax
    while abs(N_ramp*p) > gmax
        N_plt = N_plt+1;
        p = sign(area)*(abs(area)-N_plt*dt*gmax+2*dt*gmax)/(N_ramp*(N_ramp+1)*dt);
    end
    g = [0:p:N_ramp*p,sign(area)*gmax*ones(1,N_plt-2),N_ramp*p:-p:0]';
    N = N_ramp*2+N_plt;
end

if ~isempty(g)
    k = 2*pi*gamma*cumsum(g)*dt;  % (rad/cm)  % cannot use g2ks here because that will calculate cumsum(g), here we assume inherently refocus
    s = [0; diff(g)/dt];  % Gauss/cm/sec
    t = (0:dt:(N-1)*dt)';  % calculate time points
end
