function [ b,c,H,output ] = firlininf( d,N,Fp,Fs,Wp,Ws,opt)
% Design a complex FIR filter using a min-max optimization process
% Input:
% d, the desired excitation pattern
% N, the order of the FIR filter
% Fp,Fs, the position of the passband and the stopband
% Wp, Ws, the weight of the passband and the stopband
% Output:
% b, the coefficients of the designed of FIR filter
% c, the coefficients of the designed of FIR filter using cosine and sine
% as basis functions
% output: additional parameters
%

if ~exist('opt','var') % default setting
    opt.design = 'real'; % real design
end
if ~isfield(opt,'quiet')
    opt.quiet = 1;
end
% Define FpR, FpL, FsR, FsL
if length(Fp(:)) == 1
    FpR = Fp;
    FpL = -Fp;
    FsR = Fs;
    FsL = -Fs;
else
    FpL = Fp(1);
    FpR = Fp(2);
    FsL = Fs(1);
    FsR = Fs(2);
end

% Calculate the weigthing matrix
M = length(d);
w = -pi:(2*pi/M):(pi-(2*pi/M));
W_diag = zeros(M,1);
W_diag((w/2/pi<FpR) & (w/2/pi>FpL)) = Wp;
W_diag((w/2/pi>FsR) | (w/2/pi<FsL)) = Ws;
W = spdiags(W_diag,0,M,M);

% Calculate the design matrix
[~,A] = firmat(N,w);
if strcmp(opt.design,'even') % if even symmetry, only use the cosine part of the A matrix
    A = A(:,1:(N-1)/2+1);
end

% solve the min-max problem
cvx_clear;
if opt.quiet == 1
    cvx_quiet(true);
else
    cvx_quiet(false);
end
if strcmp(opt.design,'even')
    d = real(d);
    A = real(A);
    cvx_begin
    variable c((N-1)/2+1)
    minimize(norm((W*A)*c - (W*d),Inf));
    cvx_end
elseif strcmp(opt.design,'real') % complex design
    d = real(d);
    A = real(A);
    cvx_begin
    variable c(N)
    minimize(norm((W*A)*c - (W*d),Inf));
    cvx_end
elseif strcmp(opt.design,'complex') % complex design
    cvx_begin
    variable c(N) complex
    minimize(norm((W*A)*c - (W*d),Inf));
    cvx_end
end

% Calculate the achieved frequency response
H = A*c;

% Calculate the FIR filter coefficients from c
if strcmp(opt.design,'even')
    temp = zeros(N,1);
    temp(1:(N-1)/2+1) = c;
    b = firc2b(temp);
elseif strcmp(opt.design,'complex') | strcmp(opt.design,'real')
    b = firc2b(c);
end

% CVX information
output.status = cvx_status;
output.optval = cvx_optval;
output.slvitr = cvx_slvitr;
output.slvtol = cvx_slvtol;
output.cputime = cvx_cputime;
end

