function [g_total,ind_plateau,g_attack,g_decay,s_total] = ramp_for_trap(g_plateau,smax,dt)
% ramp_for_trap
% Build the attack and decay for a given plateau of a trapezoid, subject to
% the maximum slew rate constraint.
% Input
%  g_plateau: a vector recording the gradient waveform of the plateau 
%             (g_plateau(1) can be different from g_plateau(end)) (Gauss/cm)
%  smax: max slew rate (gauss/cm/sec) [14500]
%  dt: time interval between two neighboring samples (sec) [4e-6]
% Output
%  g_attack, g_decay, g_total: attack, decay, or total
%  (attack+plateau+decay) gradient waveforms
%  ind_plateau: 2x1 array where ind_plateau(1) is the starting index of
%  plateau in g_total and ind_plateau(2) is the ending index
%  s_total: slew rate waveform


%% Set defaults
if nargin < 2
    smax = 14500;
end

if nargin < 3
    dt = 4e-6;
end
g_plateau = g_plateau(:);


%% Build attack waveform
g_sign_attack = sign(g_plateau(1));
g_tgt_attack = abs(g_plateau(1));
N_attack = ceil(g_tgt_attack/(smax*dt));
s_attack = g_tgt_attack/(N_attack*dt);
g_attack = g_sign_attack*(0:s_attack*dt:(N_attack-1)*s_attack*dt)';


%% Build decay waveform
g_sign_decay = sign(g_plateau(end));
g_tgt_decay = abs(g_plateau(end));
N_decay = ceil(g_tgt_decay/(smax*dt));
s_decay = g_tgt_decay/(N_decay*dt);
g_decay = g_sign_decay*((N_decay-1)*s_decay*dt:-s_decay*dt:0)';


%% Concatenate waveforms
g_total = [g_attack;g_plateau;g_decay];
[ktemp,s_total] = g2ks(g_total,dt);
ind_plateau(1) = length(g_attack)+1;
ind_plateau(2) = length(g_attack)+length(g_plateau);
